package app

import (
	"context"
	"errors"
	"gitea.pena/PenaDevops/smtpbiz-exporter/internal/client"
	"gitea.pena/PenaDevops/smtpbiz-exporter/internal/controller/metrics"
	"gitea.pena/PenaDevops/smtpbiz-exporter/internal/initialize"
	"gitea.pena/PenaDevops/smtpbiz-exporter/internal/server/http"
	"gitea.pena/PenaDevops/smtpbiz-exporter/internal/worker/research"
	"gitea.pena/PenaDevops/smtpbiz-exporter/pkg/closer"
	"go.uber.org/zap"
	"time"
)

func Run(ctx context.Context, logger *zap.Logger, cfg initialize.Config) error {
	defer func() {
		if r := recover(); r != nil {
			logger.Error("Recovered from a panic", zap.Any("error", r))
		}
	}()

	ctx, cancel := context.WithCancel(ctx)
	defer cancel()

	shutdownGroup := closer.NewCloserGroup()

	smtpClient := client.NewSMTPClient(cfg.SmtpApiUrl, cfg.SmtpApiKey)

	researchWorker := research.NewResearch(research.Deps{
		Client:    smtpClient,
		Logger:    logger,
		Frequency: cfg.Frequency,
	})

	go researchWorker.Start(ctx)

	prometheusController := metrics.NewPrometheus(researchWorker)

	adminServer := http.NewServer(http.ServerConfig{
		Logger:      logger,
		Controllers: []http.Controller{prometheusController},
	})

	go func() {
		if err := adminServer.Start(cfg.AdminHttpURL); err != nil {
			logger.Error("failed to start admin http server", zap.Error(err))
			cancel()
		}
	}()
	adminServer.ListRoutes()

	shutdownGroup.Add(closer.CloserFunc(adminServer.Shutdown))

	<-ctx.Done()

	timeoutCtx, timeoutCancel := context.WithTimeout(context.Background(), 10*time.Second)
	defer timeoutCancel()
	if err := shutdownGroup.Call(timeoutCtx); err != nil {
		if errors.Is(err, context.DeadlineExceeded) {
			logger.Error("Shutdown timed out", zap.Error(err))
		} else {
			logger.Error("Failed to shutdown services gracefully", zap.Error(err))
		}
		return err
	}

	logger.Info("Application has stopped")
	return nil
}
